@@ -88,6 +88,7 @@ cases: list[ParameterSet | tuple[str, str]] = [
             'plugin_success_baseConfig.py',
             'plugin_fail_baseConfig.py',
             'pydantic_settings.py',
+            'decorator_implicit_classmethod.py',
         ],
     ),
     # Strict plugin config
@@ -111,6 +112,7 @@ cases: list[ParameterSet | tuple[str, str]] = [
         ('mypy-plugin.ini', 'generics.py'),
         ('mypy-plugin.ini', 'root_models.py'),
         ('mypy-plugin.ini', 'plugin_strict_fields.py'),
+        ('mypy-plugin.ini', 'final_with_default.py'),
         ('mypy-plugin-strict-no-any.ini', 'dataclass_no_any.py'),
         ('mypy-plugin-very-strict.ini', 'metaclass_args.py'),
         ('pyproject-plugin-no-strict-optional.toml', 'no_strict_optional.py'),
@@ -187,7 +189,7 @@ def test_mypy_results(config_filename: str, python_filename: str, request: pytes
         '--show-error-codes',
         '--show-traceback',
     ]
-    print(f"\nExecuting: mypy {' '.join(command)}")  # makes it easier to debug as necessary
+    print(f'\nExecuting: mypy {" ".join(command)}')  # makes it easier to debug as necessary
     mypy_out, mypy_err, mypy_returncode = mypy_api.run(command)
 
     # Need to strip filenames due to differences in formatting by OS
@@ -229,7 +231,7 @@ def test_bad_toml_config() -> None:
     full_filename = 'tests/mypy/modules/generics.py'  # File doesn't matter
 
     command = [full_filename, '--config-file', full_config_filename, '--show-error-codes']
-    print(f"\nExecuting: mypy {' '.join(command)}")  # makes it easier to debug as necessary
+    print(f'\nExecuting: mypy {" ".join(command)}')  # makes it easier to debug as necessary
     with pytest.raises(ValueError) as e:
         mypy_api.run(command)
 
