@@ -1,6 +1,6 @@
 import re
 import typing
-from typing import Any, Callable, List
+from typing import Any, Callable
 
 import pytest
 import typing_extensions
@@ -34,14 +34,14 @@ def make_title(name: str, _):
     return re.sub(r'(?<=[a-z])([A-Z])', r' \1', _capitalize(name))
 
 
-FIELD_TITLE_GENERATORS: List[Callable[[str, Any], str]] = [
+FIELD_TITLE_GENERATORS: list[Callable[[str, Any], str]] = [
     lambda t, _: t.lower(),
     lambda t, _: t * 2,
     lambda t, _: 'My Title',
     make_title,
 ]
 
-MODEL_TITLE_GENERATORS: List[Callable[[Any], str]] = [
+MODEL_TITLE_GENERATORS: list[Callable[[Any], str]] = [
     lambda m: m.__name__.upper(),
     lambda m: m.__name__ * 2,
     lambda m: 'My Model',
