@@ -1,5 +1,5 @@
 from collections import namedtuple
-from typing import Generic, NamedTuple, Optional, Tuple, TypeVar
+from typing import Generic, NamedTuple, Optional, TypeVar
 
 import pytest
 from typing_extensions import NamedTuple as TypingExtensionsNamedTuple
@@ -62,7 +62,7 @@ def test_namedtuple_schema():
     class Model(BaseModel):
         pos1: Position1
         pos2: Position2
-        pos3: Tuple[int, int]
+        pos3: tuple[int, int]
 
     assert Model.model_json_schema() == {
         'title': 'Model',
@@ -136,6 +136,24 @@ def test_namedtuple_postponed_annotation():
         Model.model_validate({'t': [-1]})
 
 
+def test_namedtuple_different_module(create_module) -> None:
+    """https://github.com/pydantic/pydantic/issues/10336"""
+
+    @create_module
+    def other_module():
+        from typing import NamedTuple
+
+        TestIntOtherModule = int
+
+        class Tup(NamedTuple):
+            f: 'TestIntOtherModule'
+
+    class Model(BaseModel):
+        tup: other_module.Tup
+
+    assert Model(tup={'f': 1}).tup.f == 1
+
+
 def test_namedtuple_arbitrary_type():
     class CustomClass:
         pass
@@ -172,7 +190,7 @@ def test_recursive_namedtuple():
         {
             'input': 'a',
             'loc': ('y', 'y', 'x'),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
@@ -195,7 +213,7 @@ def test_recursive_generic_namedtuple():
         {
             'input': 'a',
             'loc': ('y', 'y', 'x'),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
