@@ -4,11 +4,11 @@ import re
 import sys
 from datetime import datetime, timezone
 from functools import partial
-from typing import Any, List, Literal, Tuple, Union
+from typing import Annotated, Any, Generic, Literal, TypeVar, Union
 
 import pytest
 from pydantic_core import ArgsKwargs
-from typing_extensions import Annotated, Required, TypedDict, Unpack
+from typing_extensions import Required, TypedDict, Unpack
 
 from pydantic import (
     AfterValidator,
@@ -230,7 +230,7 @@ def test_kwargs():
         {
             'input': 'x',
             'loc': ('b',),
-            'msg': 'Input should be a valid integer, unable to parse string as an ' 'integer',
+            'msg': 'Input should be a valid integer, unable to parse string as an integer',
             'type': 'int_parsing',
         }
     ]
@@ -351,6 +351,20 @@ def test_unpacked_typed_dict_kwargs() -> None:
         assert exc.value.errors()[0]['loc'] == ('b',)
 
 
+def test_unpacked_generic_typed_dict_kwargs() -> None:
+    T = TypeVar('T')
+
+    class TD(TypedDict, Generic[T]):
+        t: T
+
+    @validate_call
+    def foo(**kwargs: Unpack[TD[int]]):
+        pass
+
+    with pytest.raises(ValidationError):
+        foo(t='not_an_int')
+
+
 def test_unpacked_typed_dict_kwargs_functional_syntax() -> None:
     TD = TypedDict('TD', {'in': int, 'x-y': int})
 
@@ -485,7 +499,7 @@ def test_async():
 
 def test_string_annotation():
     @validate_call
-    def foo(a: 'List[int]', b: 'float'):
+    def foo(a: 'list[int]', b: 'float'):
         return f'a={a!r} b={b!r}'
 
     assert foo([1, 2, 3], 22) == 'a=[1, 2, 3] b=22.0'
@@ -506,7 +520,7 @@ def test_string_annotation():
 
 
 def test_local_annotation():
-    ListInt = List[int]
+    ListInt = list[int]
 
     @validate_call
     def foo(a: ListInt):
@@ -609,7 +623,7 @@ def test_json_schema():
     with pytest.raises(
         PydanticInvalidForJsonSchema,
         match=(
-            'Unable to generate JSON schema for arguments validator ' 'with positional-only and keyword-only arguments'
+            'Unable to generate JSON schema for arguments validator with positional-only and keyword-only arguments'
         ),
     ):
         TypeAdapter(foo).json_schema()
@@ -693,7 +707,7 @@ def test_config_arbitrary_types_allowed():
 
 def test_config_strict():
     @validate_call(config=dict(strict=True))
-    def foo(a: int, b: List[str]):
+    def foo(a: int, b: list[str]):
         return f'{a}, {b[0]}'
 
     assert foo(1, ['bar', 'foobar']) == '1, bar'
@@ -833,8 +847,8 @@ def test_use_of_alias():
     assert foo(b=10) == 30
 
 
-def test_populate_by_name():
-    @validate_call(config=dict(populate_by_name=True))
+def test_validate_by_name():
+    @validate_call(config=dict(validate_by_name=True))
     def foo(a: Annotated[int, Field(alias='b')], c: Annotated[int, Field(alias='d')]):
         return a + c
 
@@ -921,7 +935,7 @@ def test_model_as_arg() -> None:
         y: int
 
     @validate_call(validate_return=True)
-    def f1(m1: Model1, m2: Model2) -> Tuple[Model1, Model2]:
+    def f1(m1: Model1, m2: Model2) -> tuple[Model1, Model2]:
         return (m1, m2.model_dump())  # type: ignore
 
     res = f1({'x': '1'}, {'y': '2'})  # type: ignore
@@ -1244,3 +1258,19 @@ def test_uses_local_ns():
         assert bar({'z': 1}) == M2(z=1)
 
     foo()
+
+
+# The class needs to be defined at the module level
+# For 'DeferBuildClass' to resolve:
+class DeferBuildClass(BaseModel):
+    @classmethod
+    @validate_call(config={'defer_build': True})
+    def cls_meth(cls, x: int) -> 'DeferBuildClass':
+        return DeferBuildClass()
+
+
+def test_validate_call_defer_build() -> None:
+    DeferBuildClass.cls_meth(x=1)
+
+    with pytest.raises(ValidationError):
+        DeferBuildClass.cls_meth(x='not_an_int')
